@@ -41,9 +41,13 @@ class AwsBackend(Backend):
                                 region_name=self.backend_config.region_name)
         return session.client("sts")
 
-    def configure(self, silent: bool) -> bool:
-        return config.configure(self._iam_client(), self._s3_client(), self.backend_config.bucket_name,
-                                self.backend_config.region_name, silent)
+    def validate_bucket(self) -> bool:
+        return config.validate_bucket(self._s3_client(), self.backend_config.bucket_name,
+                                      self.backend_config.region_name)
+
+    def configure(self):
+        config.configure(self._ec2_client(), self._iam_client(), self.backend_config.bucket_name,
+                         self.backend_config.subnet_id)
 
     def create_run(self, repo_user_name: str, repo_name: str) -> str:
         return runs.create_run(self._s3_client(), self._logs_client(), self.backend_config.bucket_name,
@@ -52,7 +56,8 @@ class AwsBackend(Backend):
     def submit_job(self, job: Job, counter: List[int]):
         jobs.create_job(self._s3_client(), self.backend_config.bucket_name, job, counter)
         runners.run_job(self._secretsmanager_client(), self._logs_client(), self._ec2_client(), self._iam_client(),
-                        self._s3_client(), self.backend_config.bucket_name, self.backend_config.region_name, job)
+                        self._s3_client(), self.backend_config.bucket_name, self.backend_config.region_name,
+                        self.backend_config.subnet_id, job)
 
     def get_job(self, repo_user_name: str, repo_name: str, job_id: str) -> Optional[Job]:
         return jobs.get_job(self._s3_client(), self.backend_config.bucket_name, repo_user_name, repo_name, job_id)
@@ -65,10 +70,10 @@ class AwsBackend(Backend):
         return jobs.list_jobs(self._s3_client(), self.backend_config.bucket_name, repo_user_name, repo_name,
                               run_name)
 
-    def run_job(self, job: Job) -> Runner:
-        return runners.run_job(self._secretsmanager_client(), self._logs_client(), self._ec2_client(),
-                               self._iam_client(), self._s3_client(), self.backend_config.bucket_name,
-                               self.backend_config.region_name, job)
+    def run_job(self, job: Job):
+        runners.run_job(self._secretsmanager_client(), self._logs_client(), self._ec2_client(),
+                        self._iam_client(), self._s3_client(), self.backend_config.bucket_name,
+                        self.backend_config.region_name, self.backend_config.subnet_id, job)
 
     def stop_job(self, repo_user_name: str, repo_name: str, job_id: str, abort: bool):
         runners.stop_job(self._ec2_client(), self._s3_client(), self.backend_config.bucket_name, repo_user_name,
