@@ -9,7 +9,7 @@ import yaml
 from botocore.client import BaseClient
 
 from dstack import version
-from dstack.aws import jobs, secrets, logs
+from dstack.aws import jobs, secrets, logs, local
 from dstack.backend import InstanceType, RequestStatus, RequestHead
 from dstack.jobs import Job, JobStatus, Requirements
 from dstack.runners import Resources, Runner, Gpu
@@ -24,8 +24,9 @@ def _serialize_runner(runner: Runner) -> dict:
         "gpus": [{
             "name": gpu.name,
             "memory_mib": gpu.memory_mib,
-        } for gpu in runner.resources.gpus],
+        } for gpu in (runner.resources.gpus or [])],
         "interruptible": runner.resources.interruptible is True,
+        "local": runner.resources.local is True,
     }
     data = {
         "runner_id": runner.runner_id,
@@ -40,12 +41,13 @@ def _serialize_runner(runner: Runner) -> dict:
 def _unserialize_runner(data: dict) -> Runner:
     return Runner(
         data["runner_id"],
-        data["request_id"],
+        data.get("request_id"),
         Resources(
             data["resources"]["cpus"],
             data["resources"]["memory_mib"],
             [Gpu(g["name"], g["memory_mib"]) for g in data["resources"]["gpus"]],
             data["resources"]["interruptible"] is True,
+            data["resources"].get("local") is True,
         ),
         jobs.unserialize_job(data["job"]),
         data.get("secret_names") or [],
@@ -79,6 +81,7 @@ def _get_instance_types(ec2_client: BaseClient) -> List[InstanceType]:
                     instance_type["MemoryInfo"]["SizeInMiB"],
                     reduce(list.__add__, gpus) if gpus else [],
                     "spot" in instance_type["SupportedUsageClasses"],
+                    False,
                 )
             ))
 
@@ -102,37 +105,39 @@ def _get_instance_types(ec2_client: BaseClient) -> List[InstanceType]:
     return sorted(instance_types, key=cmp_to_key(compare))
 
 
+def _matches(resources: Resources, requirements: Optional[Requirements]) -> bool:
+    if not requirements:
+        return True
+    if requirements.cpus and requirements.cpus > resources.cpus:
+        return False
+    if requirements.memory_mib and requirements.memory_mib > resources.memory_mib:
+        return False
+    if requirements.gpus:
+        gpu_count = requirements.gpus.count or 1
+        if gpu_count > len(resources.gpus or []):
+            return False
+        if requirements.gpus.name and gpu_count > len(
+                list(filter(lambda gpu: gpu.name == requirements.gpus.name,
+                            resources.gpus or []))):
+            return False
+        if requirements.gpus.memory_mib and gpu_count > len(
+                list(filter(lambda gpu: gpu.memory_mib >= requirements.gpus.memory_mib,
+                            resources.gpus or []))):
+            return False
+        if requirements.interruptible and not resources.interruptible:
+            return False
+    return True
+
+
 def _get_instance_type(ec2_client: BaseClient, requirements: Optional[Requirements]) -> Optional[InstanceType]:
     instance_types = _get_instance_types(ec2_client)
 
-    def matches(resources: Resources):
-        if not requirements:
-            return True
-        if requirements.cpus and requirements.cpus > resources.cpus:
-            return False
-        if requirements.memory_mib and requirements.memory_mib > resources.memory_mib:
-            return False
-        if requirements.gpus:
-            gpu_count = requirements.gpus.count or 1
-            if gpu_count > len(resources.gpus or []):
-                return False
-            if requirements.gpus.name and gpu_count > len(
-                    list(filter(lambda gpu: gpu.name == requirements.gpus.name,
-                                resources.gpus or []))):
-                return False
-            if requirements.gpus.memory_mib and gpu_count > len(
-                    list(filter(lambda gpu: gpu.memory_mib >= requirements.gpus.memory_mib,
-                                resources.gpus or []))):
-                return False
-            if requirements.interruptible and not resources.interruptible:
-                return False
-        return True
-
-    instance_type = next((instance_type for instance_type in instance_types if matches(instance_type.resources)), None)
+    instance_type = next(
+        (instance_type for instance_type in instance_types if _matches(instance_type.resources, requirements)), None)
     return InstanceType(instance_type.instance_name,
                         Resources(instance_type.resources.cpus, instance_type.resources.memory_mib,
                                   instance_type.resources.gpus,
-                                  requirements and requirements.interruptible)) if instance_type else None
+                                  requirements and requirements.interruptible, False)) if instance_type else None
 
 
 def _create_runner(logs_client: BaseClient, s3_client: BaseClient, bucket_name: str, runner: Runner):
@@ -153,8 +158,9 @@ def _update_runner(s3_client: BaseClient, bucket_name: str, runner: Runner):
     s3_client.put_object(Body=yaml.dump(_serialize_runner(runner)), Bucket=bucket_name, Key=key, Metadata=metadata)
 
 
-def _get_security_group_id(ec2_client: BaseClient, bucket_name: str):
-    security_group_name = "dstack_security_group_" + bucket_name.replace("-", "_").lower()
+def get_security_group_id(ec2_client: BaseClient, bucket_name: str, subnet_id: Optional[str]):
+    _subnet_postfix = ((subnet_id.replace("-", "_") + "_") if subnet_id else "")
+    security_group_name = "dstack_security_group_" + _subnet_postfix + bucket_name.replace("-", "_").lower()
     if not version.__is_release__:
         security_group_name += "_stgn"
     response = ec2_client.describe_security_groups(
@@ -170,6 +176,10 @@ def _get_security_group_id(ec2_client: BaseClient, bucket_name: str):
     if response.get("SecurityGroups"):
         security_group_id = response["SecurityGroups"][0]["GroupId"]
     else:
+        group_specification = {}
+        if subnet_id:
+            subnets_response = ec2_client.describe_subnets(SubnetIds=[subnet_id])
+            group_specification["VpcId"] = subnets_response["Subnets"][0]["VpcId"]
         security_group = ec2_client.create_security_group(
             Description="Generated by dstack",
             GroupName=security_group_name,
@@ -187,7 +197,8 @@ def _get_security_group_id(ec2_client: BaseClient, bucket_name: str):
                         },
                     ],
                 },
-            ]
+            ],
+            **group_specification
         )
         security_group_id = security_group["GroupId"]
         ip_permissions = [
@@ -248,6 +259,8 @@ def _serialize_runner_yaml(runner_id: str,
             s += f"    - name: {gpu.name}\\n      memory_mib: {gpu.memory_mib}\\n"
     if resources.interruptible:
         s += "  interruptible: true\\n"
+    if resources.local:
+        s += "  local: true\\n"
     return s[:-2]
 
 
@@ -388,23 +401,28 @@ def instance_profile_arn(iam_client: BaseClient, bucket_name: str) -> str:
                     },
                 ],
             )
-            instance_profile_arn = response["InstanceProfile"]["Arn"]
+            _instance_profile_arn = response["InstanceProfile"]["Arn"]
             iam_client.add_role_to_instance_profile(
                 InstanceProfileName=_role_name,
                 RoleName=_role_name,
             )
-            return instance_profile_arn
+            return _instance_profile_arn
         else:
             raise e
 
 
 def _get_ami_image(ec2_client: BaseClient, cuda: bool) -> Tuple[str, str]:
+    ami_name = "dstack"
+    if cuda:
+        ami_name = ami_name + "-cuda-11.1"
+    if version.__is_release__:
+        ami_name = ami_name + f"-{version.__version__}"
+    else:
+        ami_name = "[stgn] " + ami_name + "-*"
     response = ec2_client.describe_images(Filters=[
         {
             'Name': 'name',
-            'Values': [
-                'dstack-*' if version.__is_release__ else '[stgn] dstack-*'
-            ]
+            'Values': [ami_name]
         },
     ], )
     images = list(filter(lambda i: cuda == ("cuda" in i["Name"]) and i["State"] == "available", response["Images"]))
@@ -412,11 +430,11 @@ def _get_ami_image(ec2_client: BaseClient, cuda: bool) -> Tuple[str, str]:
         ami = next(iter(sorted(images, key=lambda i: i["CreationDate"], reverse=True)))
         return ami["ImageId"], ami["Name"]
     else:
-        raise Exception(f"Can't find an AMI image 'dstack-*' (cuda={cuda})")
+        raise Exception(f"Can't find an AMI image '{ami_name}")
 
 
 def _run_instance(ec2_client: BaseClient, iam_client: BaseClient, bucket_name: str, region_name: str,
-                  runner_id: str, instance_type: InstanceType) -> str:
+                  subnet_id: Optional[str], runner_id: str, instance_type: InstanceType) -> str:
     launch_specification = {}
     if not version.__is_release__:
         launch_specification["KeyName"] = "stgn_dstack"
@@ -428,6 +446,17 @@ def _run_instance(ec2_client: BaseClient, iam_client: BaseClient, bucket_name: s
                 "InstanceInterruptionBehavior": "stop",
             }
         }
+    if subnet_id:
+        launch_specification["NetworkInterfaces"] = [
+            {
+                "AssociatePublicIpAddress": True,
+                "DeviceIndex": 0,
+                "SubnetId": subnet_id,
+                "Groups": [get_security_group_id(ec2_client, bucket_name, subnet_id)],
+            },
+        ]
+    else:
+        launch_specification["SecurityGroupIds"] = [get_security_group_id(ec2_client, bucket_name, subnet_id)]
     response = ec2_client.run_instances(
         BlockDeviceMappings=[
             {
@@ -442,7 +471,6 @@ def _run_instance(ec2_client: BaseClient, iam_client: BaseClient, bucket_name: s
         InstanceType=instance_type.instance_name,
         MinCount=1,
         MaxCount=1,
-        SecurityGroupIds=[_get_security_group_id(ec2_client, bucket_name)],
         IamInstanceProfile={
             "Arn": instance_profile_arn(iam_client, bucket_name),
         },
@@ -482,16 +510,17 @@ def _run_instance(ec2_client: BaseClient, iam_client: BaseClient, bucket_name: s
 
 
 def _run_instance_retry(ec2_client: BaseClient, iam_client: BaseClient, bucket_name: str, region_name: str,
-                        runner_id: str, instance_type: InstanceType, attempts: int = 3) -> str:
+                        subnet_id: Optional[str], runner_id: str, instance_type: InstanceType, attempts: int = 3) \
+        -> str:
     try:
-        return _run_instance(ec2_client, iam_client, bucket_name, region_name, runner_id,
+        return _run_instance(ec2_client, iam_client, bucket_name, region_name, subnet_id, runner_id,
                              instance_type)
     except Exception as e:
         if hasattr(e, "response") and e.response.get("Error") and e.response["Error"].get(
                 "Code") == "InvalidParameterValue":
             if attempts > 0:
                 time.sleep(CREATE_INSTANCE_RETRY_RATE_SECS)
-                return _run_instance_retry(ec2_client, iam_client, bucket_name, region_name, runner_id,
+                return _run_instance_retry(ec2_client, iam_client, bucket_name, region_name, subnet_id, runner_id,
                                            instance_type, attempts - 1)
             else:
                 raise Exception("Failed to retry", e)
@@ -500,33 +529,41 @@ def _run_instance_retry(ec2_client: BaseClient, iam_client: BaseClient, bucket_n
 
 
 def run_job(secretsmanager_client: BaseClient, logs_client: BaseClient, ec2_client: BaseClient, iam_client: BaseClient,
-            s3_client: BaseClient,
-            bucket_name: str, region_name, job: Job) -> Runner:
+            s3_client: BaseClient, bucket_name: str, region_name, subnet_id: Optional[str], job: Job):
     if job.status == JobStatus.SUBMITTED:
-        instance_type = _get_instance_type(ec2_client, job.requirements)
-        if instance_type:
-            runner_id = uuid.uuid4().hex
-            job.runner_id = runner_id
+        runner = None
+        try:
+            job.runner_id = uuid.uuid4().hex
             jobs.update_job(s3_client, bucket_name, job)
             secret_names = secrets.list_secret_names(secretsmanager_client, bucket_name)
-            runner = Runner(runner_id, None, instance_type.resources, job, secret_names)
-            _create_runner(logs_client, s3_client, bucket_name, runner)
-            try:
-                request_id = _run_instance_retry(ec2_client, iam_client, bucket_name, region_name, runner_id,
-                                                 instance_type)
-                runner.request_id = request_id
-                job.request_id = request_id
-                jobs.update_job(s3_client, bucket_name, job)
-                _update_runner(s3_client, bucket_name, runner)
-                return runner
-            except Exception as e:
-                job.status = JobStatus.FAILED
-                jobs.update_job(s3_client, bucket_name, job)
-                raise e
-        else:
+            if job.requirements and job.requirements.local:
+                resources = local.check_runner_resources(job.runner_id)
+                if _matches(resources, job.requirements):
+                    runner = Runner(job.runner_id, None, resources, job, secret_names)
+                    _create_runner(logs_client, s3_client, bucket_name, runner)
+                    runner.request_id = local.start_runner_process(job.runner_id)
+                else:
+                    job.status = JobStatus.FAILED
+                    jobs.update_job(s3_client, bucket_name, job)
+                    sys.exit(f"Local resources do not match requirements")
+            else:
+                instance_type = _get_instance_type(ec2_client, job.requirements)
+                runner = Runner(job.runner_id, None, instance_type.resources, job, secret_names)
+                _create_runner(logs_client, s3_client, bucket_name, runner)
+                if instance_type:
+                    runner.request_id = _run_instance_retry(ec2_client, iam_client, bucket_name, region_name, subnet_id,
+                                                            job.runner_id, instance_type)
+                else:
+                    job.status = JobStatus.FAILED
+                    job.request_id = runner.request_id
+                    jobs.update_job(s3_client, bucket_name, job)
+                    sys.exit(f"No instance type matching requirements")
+            _update_runner(s3_client, bucket_name, runner)
+        except Exception as e:
             job.status = JobStatus.FAILED
+            job.request_id = runner.request_id if runner else None
             jobs.update_job(s3_client, bucket_name, job)
-            sys.exit(f"No instance type matching requirements")
+            raise e
     else:
         raise Exception("Can't create a request for a job which status is not SUBMITTED")
 
@@ -573,12 +610,26 @@ def _terminate_instance(ec2_client: BaseClient, request_id: str):
             raise e
 
 
-def get_request_head(ec2_client: BaseClient, job: Job) -> RequestHead:
+def get_request_head(ec2_client: BaseClient, s3_client: BaseClient, bucket_name: str,
+                     job: Job, runner: Optional[Runner] = None) -> RequestHead:
+    _local = job.requirements and job.requirements.local
     interruptible = job.requirements and job.requirements.interruptible
+    request_id = None
     if job.request_id:
-        if interruptible:
+        request_id = job.request_id
+    elif runner and runner.request_id:
+        request_id = runner.request_id
+    elif not runner:
+        runner = _get_runner(s3_client, bucket_name, job.runner_id)
+        if runner:
+            request_id = runner.request_id
+    if request_id:
+        if _local:
+            _running = local.is_running(request_id)
+            return RequestHead(job.job_id, RequestStatus.RUNNING if _running else RequestStatus.TERMINATED, None)
+        elif interruptible:
             try:
-                response = ec2_client.describe_spot_instance_requests(SpotInstanceRequestIds=[job.request_id])
+                response = ec2_client.describe_spot_instance_requests(SpotInstanceRequestIds=[request_id])
                 if response.get("SpotInstanceRequests"):
                     status = response["SpotInstanceRequests"][0]["Status"]
                     if status["Code"] in ["fulfilled", "request-canceled-and-instance-running"]:
@@ -607,7 +658,7 @@ def get_request_head(ec2_client: BaseClient, job: Job) -> RequestHead:
                     raise e
         else:
             try:
-                response = ec2_client.describe_instances(InstanceIds=[job.request_id])
+                response = ec2_client.describe_instances(InstanceIds=[request_id])
                 if response.get("Reservations") and response["Reservations"][0].get("Instances"):
                     state = response["Reservations"][0]["Instances"][0]["State"]
                     if state["Name"] in ["running"]:
@@ -628,13 +679,16 @@ def get_request_head(ec2_client: BaseClient, job: Job) -> RequestHead:
                 else:
                     raise e
     else:
-        message = "The spot instance request ID is not specified" if interruptible else "The instance ID is not specified"
+        message = "The spot instance request ID is not specified" if interruptible \
+            else "The instance ID is not specified"
         return RequestHead(job.job_id, RequestStatus.TERMINATED, message)
 
 
 def _stop_runner(ec2_client: BaseClient, s3_client: BaseClient, bucket_name: str, runner: Runner):
     if runner.request_id:
-        if runner.resources.interruptible:
+        if runner.resources.local:
+            local.stop_process(runner.request_id)
+        elif runner.resources.interruptible:
             _cancel_spot_request(ec2_client, runner.request_id)
         else:
             _terminate_instance(ec2_client, runner.request_id)
@@ -645,8 +699,9 @@ def stop_job(ec2_client: BaseClient, s3_client: BaseClient, bucket_name: str, re
              job_id: str, abort: bool):
     job_head = jobs.list_job_head(s3_client, bucket_name, repo_user_name, repo_name, job_id)
     job = jobs.get_job(s3_client, bucket_name, repo_user_name, repo_name, job_id)
-    request_status = get_request_head(ec2_client, job).status if job else RequestStatus.TERMINATED
     runner = _get_runner(s3_client, bucket_name, job.runner_id) if job else None
+    request_status = get_request_head(ec2_client, s3_client, bucket_name, job,
+                                      runner).status if job else RequestStatus.TERMINATED
     if job_head and job_head.status.is_unfinished() or job and job.status.is_unfinished() \
             or runner and runner.job.status.is_unfinished() \
             or request_status != RequestStatus.TERMINATED:
